<?php
namespace Drupal\editoria11y\Controller;

use Drupal;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\editoria11y\Dashboard;
use Drupal\user\Entity\User;
use Drupal\Core\Routing\RouteObjectInterface;
use Drupal\Core\Form\FormState;

/**
 * Provides route responses for the Editoria11y module.
 */
class DashboardController extends ControllerBase
{
  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
    protected $account;

  /**
   * The current database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
    protected $connection;

  /**
   * @var Dashboard
   */
    protected $dashboard;

  /**
   * @param $dashboard
   */
    public function __construct($dashboard)
    {
        $this->dashboard = $dashboard;
    }

  /**
   * {@inheritdoc}
   */
    public static function create($container)
    {
        return new static(
            $container->get('editoria11y.dashboard')
        );
    }

    /**
     * Throws error if sync is disabled.
     */
    private function sync_status() {
      $config = \Drupal::config('editoria11y.settings');
      // force standard bool
      $sync = $config->get('disable_sync');
      if (!!$sync) {
        $msg = t("Dashboard sync is disabled in Editoria11y configuration.");
        \Drupal::messenger()->addWarning($msg, $type = 'warning');
      }
    }


    
    /**
     * Get lang from URL query or Drupal native code.
     *
     * @return string
     */
    

    
    /**
     * Gets list of pages with accessibility issues
     *
     * @return array
     */
    public function get_test_results()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        // load filters
        $form_state = new FormState();
        $form_state->setRebuild();
        $form = \Drupal::formBuilder()->buildForm('Drupal\editoria11y\Form\DashboardFilters', $form_state);
        $form_title = $form_state->getValue('title');
        $form_url = $form_state->getValue('url');
        $lang = \Drupal::request()->query->get('lang');

        $header = [
        ['data' => t("Page"), 'field' => 't.page_path'],
        ['data' => t("Issues Found"), 'field' => 't.page_result_count', 'sort' => 'desc'],
        ['data' => t("Type"), 'field' => 't.entity_type'],
        ];
        $rows = [];

        $results = $this->dashboard->getResults($page, $header, 50, $form_title, $form_url);
        foreach ($results["results"] as $record) {
          // to review: https://drupal.stackexchange.com/questions/144992/how-do-i-create-a-link
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $linkToPage = $record->page_title;
            }
            $issuesByPage = Url::fromUserInput("/admin/reports/editoria11y/page?q=" . $record->page_path);
            $linkToIssuesByPage = Link::fromTextAndUrl($record->page_result_count, $issuesByPage)->toString();
            $filterByType = Url::fromUserInput("/admin/reports/editoria11y/pages?type=" . $record->entity_type . "&lang=" . $lang);
            $linkToFilterByType = Link::fromTextAndUrl($record->entity_type, $filterByType)->toString();
            $rows[] = [$linkToPage, $linkToIssuesByPage, $linkToFilterByType];
          // Todo add delete trashcan?
        }

        $render = [];    
        $render['form'] = [$form];
 
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url',
          ],
          'tags' => [
            'editoria11y.dashboard',
          ],
        ]
        ];
        $pager_manager
        ->createPager($results["count"], 50);
        $render[] = [
          '#type' => 'pager',
          '#suffix' =>  "<p>&nbsp;</p>"
        ];
          


        return $render;
    }

    /**
     * Given a query variable of a page id, gets issues on page.
     *
     * @return array
     */
    public function get_issues_by_page()
    {
        $header = [
          ['data' => t("Count"), 'field' => 't.result_name_count', 'sort' => 'desc'],
          ['data' => t("Issue"), 'field' => 't.result_name'],
        ];

        $rows = [];

        $key = Drupal::request()->query->get('q');
        if (!is_null($key)) {
          $key = Xss::filter($key);
        }
        $results = $this->dashboard->getResultsByPage($key, $header);
        $page = "";

        foreach ($results as $record) {
            $page_url = Url::fromUserInput($record->page_path);
            $page = Link::fromTextAndUrl($record->page_title, $page_url)->toString();
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . $record->result_name);
            $pages_by_issue = Link::fromTextAndUrl($record->result_name, $pages_by_issue_url)->toString();
            $rows[] = [$record->result_name_count, $pages_by_issue];
        }

        if (\Drupal::currentUser()->hasPermission('administer editoria11y checker')) {
          $purge = "<a href='#ed11y-purge' class='ed11y-reset-page action-link action-link--danger action-link--icon-trash' data-target='page'> ". t("Reset results for this page") . "</a>";
        }

        return [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2 id='ed11y-page'>".t('Page: ') . $page ."</h2>",
        ]
        ],
        
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        'purger' => [
          'element' => [
            '#markup' => $purge,
          ]
          ],
        ];
    }

    /**
     * Given a query variable of an issue name, builds table of issues.
     *
     * @return array
     */
    public function get_pages_by_issue()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        $header = [
          ['data' => t("Count"), 'field' => 'page_result_count', 'sort'=>'desc'],
          ['data' => t("Page"), 'field'=>'page_path'],
        ];

        $rows = [];
        $key = Drupal::request()->query->get('q');
        $results = $this->dashboard->getResultsByIssue($key, $page, $header);

        foreach ($results["results"] as $record) {
            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $rows[] = [$record->result_name_count, $link];
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render[] = [
        '#type' => 'container',
        'page_title' => [
        'element' => [
          '#markup' => "<h2>" . t('Pages with "' . $key . '" errors') . "</h2>",
        ]
        ],
        'content' => [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ],
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

    private function get_top_pages()
    {
        $header = [
          t("Page"),
          t("Count"),
        ];
        $rows = [];
        $results = $this->dashboard->getTopResults();
        foreach ($results as $record) {
            if (strpos($record->page_path, "/") === 0) {
                $url = Url::fromUserInput($record->page_path);
                $linkToPage = Link::fromTextAndUrl($record->page_title, $url);
            } else {
                $linkToPage = $record->page_title;
            }
            $link_options = array(
              // Increase target size
              'attributes' => array(
                'class' => array(
                  'action-link--small'
                ),
              ),
            );
            $linkString = Url::fromRoute('editoria11y.reports_issues_by_page', ['q' => $record->page_path], $link_options);
            $linkToIssueList = Link::fromTextAndUrl($record->page_result_count, $linkString)->toString();
            $rows[] = [ $linkToPage, $linkToIssueList];
          // Todo add delete trashcan?
        }

        $render = [];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#rows' => $rows,
          '#prefix' => '<h2>' . t("Pages with the most issues") .'</h2>',
        '#cache' => [
          'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => t('View all pages with issues'),
          '#attributes' => ['class' => 'button button--primary'],
          '#url' => Url::fromRoute('editoria11y.reports_pages'),
          '#suffix' => '<br>&nbsp;</p>',
        ],
        ];

        $render[] = [
          'spacer' => [
            '#type' => 'html_tag',
            '#tag' => 'p',
          ],
        ];

        return $render;
    }

    /**
     * Gets summary info on recent dismissals for dashboard panel.
     *
     * @return array
     */
    private function get_recent_dismissals()
    {

        $result_service = \Drupal::service('editoria11y.dashboard');
        $lang = $this->dashboard->get_page_lang();
        $results = $result_service->getRecentDismissals($lang);

        //   ['page_title','route_name','page_path','created','updated','result_name','dismissal_status','uid','created','updated', 'stale']);
        $header = [
          t('Page'),
          t('Issue'),
          t('Marked as'),
          t('User'),
          t('Date'),
        ];

        $rows = [];

        foreach ($results as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }
            $date = \Drupal::service('date.formatter')->format($record->created, 'short');

          // todo: return username instead of user id
          // todo: show stale as better than numerical boolean
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, $name, $date];
        }

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#prefix' => '<h2 id="recent_dismissals">' . t("Recent dismissals") . '</h2>',
          '#header' => $header,
          '#rows' => $rows,
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        $render[] = [
        'link' => [
          '#prefix' => '<p>',
          '#type' => 'link',
          '#title' => t('View all dismissals'),
          '#attributes' => ['class' => 'button button--primary'],
          '#url' => Url::fromRoute('editoria11y.reports_dismissals'),
          '#suffix' => '<br>&nbsp;</p>',
        ],
        ];
        return $render;
    }


    /**
     * Builds table of top issues. Count limiter used for dashboard summary panel.
     *
     * @return array
     */
    private function get_top_issues($count) {
        $result_service = \Drupal::service('editoria11y.dashboard');
        
        $language = Drupal::request()->query->get('lang');
        if (!is_null($language)) {
          $lang = Xss::filter($language);
        } else {
          $lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
        }
        $links = $this->dashboard->get_language_filters($lang);
        
        $results = $result_service->getIssueList($count, $lang);
        
        $header = [
        t('Issue'),
        t('Count'),
        ];

        $rows = [];

        foreach ($results as $record) {
            $pages_by_issue_url = Url::fromUserInput("/admin/reports/editoria11y/issue?q=" . t($record->result_name));
            $pages_by_issue = Link::fromTextAndUrl($record->result_name, $pages_by_issue_url)->toString();
            $rows[] = [$pages_by_issue, $record->result_name_count];
        }

        $prefix = "";
        if ($count === 0 || $count > 25) {
          // Add language filters to page
          $prefix = $links;
        }
        $prefix .= "<h2>" . t("Most frequent issues") . "</h2>";

        $render = [];
        $render[] = [
        '#type' => 'container',
        'content' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
          '#prefix' => $prefix,
          '#cache' => [
            'contexts' => [
              'user',
              'url'
            ],
            'tags' => [
              'editoria11y.dashboard',
            ]
          ],
        ],
        ];
        if ($count > 0) {
          $render[] = [
            'link' => [
              '#prefix' => '<p>',
              '#type' => 'link',
              '#title' => t('View all issues'),
              '#attributes' => ['class' => 'button button--primary'],
              '#url' => Url::fromRoute('editoria11y.reports_all_issues'),
              '#suffix' => '<br>&nbsp;</p>',
            ],
            ];
        }
        return $render;
    }


    /**
     * Builds page with all dismissals.
     *
     * @return array
     */
    public function get_dismissals()
    {

        $pager_manager = Drupal::service('pager.manager');
        $page = $pager_manager
        ->findPage();

        // load filters
        $form_state = new FormState();
        $form_state->setRebuild();
        $form = \Drupal::formBuilder()->buildForm('Drupal\editoria11y\Form\DashboardFilters', $form_state);
        $form_title = $form_state->getValue('title');
        $form_url = $form_state->getValue('url');

        $header = [
          'page' => t("Page with issue"),
          'issue' => t("Dismissed issue"),
          'dismissals' => t("Marked"),
          'user' => t("By"),
          'stale' => t('Still on page'),
          'date' => t("Last check")
          
        ];

        $rows = [];

        $results = $this->dashboard->getDismissals($page, $form_title, $form_url);
        $counter = 0;
        foreach ($results["results"] as $record) {
            $user = User::load($record->uid);
            $name = $user->getDisplayName();

            if (str_starts_with($record->page_path, "/")) {
                $url = Url::fromUserInput($record->page_path);
                $link = Link::fromTextAndUrl($record->page_title, $url)->toString();
            } else {
                $link = $record->page_title;
            }

            $stale = $record -> stale ? "No" : "Yes";

            $date = \Drupal::service('date.formatter')->format($record->updated);

          // todo: return username instead of user id
            $rows[] = [$link, $record->result_name, $record -> dismissal_status, ['id' => 'uid' . $record->uid, 'data' => $name], $date, $stale];
            
        }

        $pager_manager
        ->createPager($results["count"], 50);

        $render = [];
        $render['form'] = [$form];
        $render[] = [
        '#type' => 'table',
        '#header' => $header,
        '#prefix' => '<span id="ed11y-rowpurger"></span>',
        '#rows' => $rows,
        '#cache' => [
        'contexts' => [
            'user',
            'url'
          ],
          'tags' => [
            'editoria11y.dashboard',
          ]
        ]
        ];
        $render[] = [
        '#type' => 'pager',
        ];
        return $render;
    }

  /**
   * Page: summary dashboard with three panels.
   *
   * @return array
   *   A simple renderable array.
   */
    public function dashboard(): array
    {
      $this->sync_status();

      $render = [
        '#type' => 'container',
        [
          '#type' => 'container',
          '#attributes' => array(
            'class' => 'layout-row',
          ),
          [
            '#type' => 'container',
            '#attributes' => array(
              'class' => 'layout-column layout-column--half',
            ),
            'content' => [
              $this->get_top_pages(),

            ],
          ],
          [
            '#type' => 'container',
            '#attributes' => array(
              'class' => 'layout-column layout-column--half',
            ),
            'content' => [
              $this->get_top_issues(5),
            ],
          ],
        ],
      ];
      $render[] = [
        $this->get_recent_dismissals()
      ];
      return $render;
    }

  /**
   * Page: pages with issues.
   *
   * @return array
   *   A simple renderable array.
   */
  public function results(): array
  {
    $this->sync_status();  
    return $this->get_test_results();
  }

  /**
   * Page: list of issues by type.
   *
   * @return array
   *   A simple renderable array.
   */
  public function all_issues(): array
  {
    $this->sync_status();
    return $this->get_top_issues(0);
  }

  /**
   * Page: pages with a specific issue.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function pages_by_issue(): array
    {
      $this->sync_status();
      return $this->get_pages_by_issue();
    }

  /**
   * Page: issues on a given page.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, pagination
    public function issues_by_page(): array
    {
      $this->sync_status();
      return $this->get_issues_by_page();
    }

  /**
   * Page: dismissals.
   *
   * @return array
   *   A simple renderable array.
   */
  // todo: view by page or test, filter by user
    public function dismissals(): array
    {
      $this->sync_status();
      return $this->get_dismissals();
    }
}
